﻿Class PageOrganize
    ''' <summary>
    ''' Delete the GDI Object. There is no public methode like MSDN says. This is
    ''' very important if you want to calculate several images because otherwise
    ''' you get memory leaks.
    ''' </summary>
    ''' <param name="hObject">The pointer of the Bitmap Object</param>
    ''' <returns>1 if all is ok</returns>
    ''' <remarks>http://msdn.microsoft.com/en-us/library/dd183402%28VS.85%29.aspx</remarks>
    <System.Runtime.InteropServices.DllImport("gdi32.dll")>
    Public Shared Function DeleteObject(ByVal hObject As IntPtr) As Integer
    End Function



#Region "Progressing"

    ''' <summary>
    ''' Progressing Dependency Property
    ''' </summary>
    Public Shared ReadOnly ProgressingProperty As DependencyProperty = _
        DependencyProperty.Register("Progressing", GetType(Boolean), GetType(PageOrganize), _
            New FrameworkPropertyMetadata(False, _
                AddressOf OnProgressingChanged))

    ''' <summary>
    ''' Gets or sets the Progressing property.  This dependency property 
    ''' indicates if sorting is active
    ''' </summary>
    Public Property Progressing() As Boolean
        Get
            Return CType(GetValue(ProgressingProperty), Boolean)
        End Get
        Set(ByVal value As Boolean)
            SetValue(ProgressingProperty, value)
        End Set
    End Property

    ''' <summary>
    ''' Handles changes to the Progressing property.
    ''' </summary>
    Private Overloads Shared Sub OnProgressingChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        Dim target As PageOrganize = CType(d, PageOrganize)
        Dim oldProgressing As Boolean = CType(e.OldValue, Boolean)
        Dim newProgressing As Boolean = target.Progressing
        target.OnProgressingChanged(oldProgressing, newProgressing)
    End Sub

    ''' <summary>
    ''' Provides derived classes an opportunity to handle changes to the Progressing property.
    ''' </summary>
    Protected Overridable Overloads Sub OnProgressingChanged(ByVal oldProgressing As Boolean, ByVal newProgressing As Boolean)
        If newProgressing = True Then
            Me.txtProgressing.Visibility = Windows.Visibility.Visible
        Else
            Me.txtProgressing.Visibility = Windows.Visibility.Collapsed
        End If
    End Sub

#End Region



    'The timer is used that way because without XAML it is more comportable for me
    'Time is used just any refresh gui. 
    'I did this with timer because I also wanted to update values with calculated shortest way as an option (not included yet)
    Private WithEvents TimerUpdate As System.Windows.Threading.DispatcherTimer


    ''' <summary>
    ''' Initalize some stuff 
    ''' Also load the Workerlist into the GUI
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Page_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles MyBase.Loaded
        Dim Gut As Boolean = False
        For Each item As IObject In Application.WorkList
            If TypeOf item Is clsStart Then Gut = True
        Next
        If Gut = False Then Application.WorkList.Add(New clsStart)

        Gut = False
        For Each item As IObject In Application.WorkList
            If TypeOf item Is clsEnd Then Gut = True
        Next
        If Gut = False Then Application.WorkList.Add(New clsEnd)

        Call RefreshItems()

        TimerUpdate = New System.Windows.Threading.DispatcherTimer
        TimerUpdate.Interval = TimeSpan.FromMilliseconds(500)
        TimerUpdate.Start()
    End Sub

    ''' <summary>
    ''' Add items to the GUI by using a host control to arrange items
    ''' </summary>
    Private Sub RefreshItems()
        Dim pt As Point = New Point(10, 10)
        Me.OrganizerGrid.Children.Clear()
        For Each item As IObject In Application.WorkList
            Dim Host As New PluginHostControl() With {.Title = item.Title}
            If item.Location.X = 0 And item.Location.Y = 0 Then
                item.Location = pt
                pt = New Point(pt.X + 60, pt.Y)
                If pt.X > Me.OrganizerGrid.ActualWidth - 80 Then
                    pt = New Point(10, pt.Y + 60)
                End If
            End If
            Host.img1.Source = New BitmapImage(New Uri("/Images/" & item.Title & ".png", UriKind.Relative))
            Host.MyObject = item
            Host.Margin = New Thickness(item.Location.X, item.Location.Y, 0, 0)
            Host.SnapToGridEnabled = True
            Host.SnapsToDevicePixels = 20
            Me.OrganizerGrid.Children.Add(Host)
        Next
    End Sub

    ''' <summary>
    ''' Calculate the shortest and best way
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>This works sometimes but not at all. I think others can include their optimized algorithm here</remarks>
    Private Sub btnCalculate_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnCalculate.Click
        Dim tstart = Now
        Call Sort()

        Dim tspan = (Now - tstart).TotalMilliseconds
        Call MsgBox(tspan.ToString)
    End Sub


    ''' <summary>
    ''' sort the WorkList by using distance and dependency of objects
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Sort()
        Dim cCalc As New clsSorter
        Me.Progressing = True
        For Each item As IObject In Application.WorkList
            item.Index = 0
        Next

        'calculate the distance between previous and next object
        'because only these 3 objects are in comparation, it could
        'be that the result will change after using this again.
        For i As Integer = 1 To Application.WorkList.Count - 1 - 1
            Dim D1 As Single = cCalc.GetDistance(Application.WorkList(i - 1), Application.WorkList(i + 0))
            Dim D2 As Single = cCalc.GetDistance(Application.WorkList(i - 0), Application.WorkList(i + 1))
            Dim D3 As Single = cCalc.GetDistance(Application.WorkList(i - 1), Application.WorkList(i + 1))

            If D1 < D2 Then
                If D1 < D3 Then
                    Application.WorkList(i - 1).Index -= 1
                    Application.WorkList(i - 0).Index -= 1
                    Application.WorkList(i + 1).Index += 1
                Else
                    Application.WorkList(i - 1).Index -= 1
                    Application.WorkList(i - 0).Index += 1
                    Application.WorkList(i + 1).Index -= 1
                End If
            Else
                If D2 < D3 Then
                    Application.WorkList(i - 1).Index += 1
                    Application.WorkList(i - 0).Index -= 1
                    Application.WorkList(i + 1).Index -= 1
                Else
                    Application.WorkList(i - 1).Index -= 1
                    Application.WorkList(i - 0).Index += 1
                    Application.WorkList(i + 1).Index -= 1
                End If
            End If
        Next

        'sort now the items by using default comparation
        Application.WorkList.Sort(AddressOf cCalc.Compare)
        Me.Progressing = False
    End Sub

    ''' <summary>
    ''' create path as image
    ''' </summary>
    ''' <remarks>perhaps it is also possible to do it without .Drawing reference. 
    ''' But it is that easy with it, I just used it</remarks>
    Private Sub RefreshView()
        If Me.Progressing = True Then Exit Sub
        Dim Pen As System.Drawing.Pen = New System.Drawing.Pen(System.Drawing.Brushes.Red, 8)
        Pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash

        Dim Way As New List(Of System.Drawing.PointF)
        For Each item As IObject In Application.WorkList
            Way.Add(New System.Drawing.PointF(item.Location.X, item.Location.Y))
        Next

        Dim bm As New System.Drawing.Bitmap(Me.OrganizerGrid.ActualWidth, Me.OrganizerGrid.ActualHeight, System.Drawing.Imaging.PixelFormat.Format32bppArgb)
        Using gr As System.Drawing.Graphics = System.Drawing.Graphics.FromImage(bm)
            gr.DrawCurve(Pen, Way.ToArray)
        End Using

        Me.imgResult.Source = BitmapToWpfBitmapSource(bm)

    End Sub

    ''' <summary>
    ''' map the bitmap data to the WPF source
    ''' </summary>
    ''' <param name="bmSrc">system.drawing.bitmap memory</param>
    ''' <returns>bitmapsource to use in WPF</returns>
    ''' <remarks>This methode is recommended and works 100% without memory leaks.</remarks>
    Public Function BitmapToWpfBitmapSource(ByVal bmSrc As System.Drawing.Bitmap) As BitmapSource
        If bmSrc Is Nothing Then Return Nothing
        Dim res As BitmapSource
        Dim Ptr As IntPtr
        Ptr = bmSrc.GetHbitmap(System.Drawing.Color.Black) 'Create GDI bitmap object
        res = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(Ptr, IntPtr.Zero, System.Windows.Int32Rect.Empty, BitmapSizeOptions.FromWidthAndHeight(bmSrc.Width, bmSrc.Height))
        Dim ret As Integer = DeleteObject(Ptr) 'Delete GDI bitmap object
        GC.Collect() 'Because the code is not managed, we need to call the collector manually to avoid memory spikes
        Return res
    End Function


    Private Sub TimerUpdate_Tick(sender As Object, e As System.EventArgs) Handles TimerUpdate.Tick
        Call RefreshView()
    End Sub


    ''' <summary>
    ''' Create a copy from a source object
    ''' </summary>
    ''' <param name="SrcObject">IObject</param>
    ''' <param name="DstObject">IObject</param>
    ''' <remarks>not in use but prepared</remarks>
    Private Sub CloneObject(ByVal SrcObject As Object, ByRef DstObject As Object)
        Dim mS As New IO.MemoryStream
        Dim B As New Runtime.Serialization.Formatters.Binary.BinaryFormatter
        B.Serialize(mS, SrcObject)
        mS.Seek(0, IO.SeekOrigin.Begin)
        DstObject = B.Deserialize(mS)
        mS.Close()
    End Sub
End Class
