﻿Imports System.ComponentModel
Imports System.Windows.Controls.Primitives

''' <summary>
''' Can host a plugin as graphical element
''' </summary>
Public Class PluginHostControl


#Region "IsControlSelectedProperty"

    ''' <summary>
    ''' IsControlSelectedProperty Dependency Property
    ''' </summary>
    Public Shared ReadOnly IsControlSelectedPropertyProperty As DependencyProperty = _
        DependencyProperty.Register("IsControlSelectedProperty", GetType(Boolean), GetType(PluginHostControl), _
            New FrameworkPropertyMetadata(False))

    ''' <summary>
    ''' Gets or sets the IsControlSelectedProperty property.  This dependency property 
    ''' indicates if control is selected and will show size change controls
    ''' </summary>
    Public Property IsControlSelectedProperty() As Boolean
        Get
            Return CType(GetValue(IsControlSelectedPropertyProperty), Boolean)
        End Get
        Set(ByVal value As Boolean)
            SetValue(IsControlSelectedPropertyProperty, value)
        End Set
    End Property

#End Region


    Private _MyObject As IObject
    Public Property MyObject() As IObject
        Get
            Return _MyObject
        End Get
        Set(ByVal value As IObject)
            _MyObject = value
        End Set
    End Property


#Region "Title"

    ''' <summary>
    ''' Title Dependency Property
    ''' </summary>
    Public Shared ReadOnly TitleProperty As DependencyProperty = _
        DependencyProperty.Register("Title", GetType(String), GetType(PluginHostControl), _
            New FrameworkPropertyMetadata(""))

    ''' <summary>
    ''' Gets or sets the Title property.  This dependency property 
    ''' indicates the title of the plugin
    ''' </summary>
    Public Property Title() As String
        Get
            Return CType(GetValue(TitleProperty), String)
        End Get
        Set(ByVal value As String)
            SetValue(TitleProperty, value)
        End Set
    End Property

#End Region




    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        HorizontalAlignment = Windows.HorizontalAlignment.Left
        VerticalAlignment = Windows.VerticalAlignment.Top

        RenderTransform = New RotateTransform

        Me.DataContext = Me
    End Sub



    Public Shared ReadOnly SnapToGridEnabledProperty As DependencyProperty = DependencyProperty.Register("SnapToGridEnabled", GetType(Boolean), GetType(PluginHostControl))
    Public Property SnapToGridEnabled As Boolean
        Get
            Return GetValue(SnapToGridEnabledProperty)
        End Get

        Set(ByVal value As Boolean)
            SetValue(SnapToGridEnabledProperty, value)
        End Set
    End Property


#Region "SnapPixelCountProperty"

    ''' <summary>
    ''' SnapPixelCountProperty Dependency Property
    ''' </summary>
    Public Shared ReadOnly SnapPixelCountPropertyProperty As DependencyProperty = _
        DependencyProperty.Register("SnapPixelCountProperty", GetType(Integer), GetType(PluginHostControl), _
            New FrameworkPropertyMetadata(CInt(10)))

    ''' <summary>
    ''' Gets or sets the SnapPixelCountProperty property.  This dependency property 
    ''' indicates the size of the grid to snap. Default is 10 Pixels.
    ''' </summary>
    Public Property SnapPixelCountProperty() As Integer
        Get
            Return CType(GetValue(SnapPixelCountPropertyProperty), Integer)
        End Get
        Set(ByVal value As Integer)
            SetValue(SnapPixelCountPropertyProperty, value)
        End Set
    End Property

#End Region




    Private Sub Move(ByVal sender As Object, ByVal e As DragDeltaEventArgs) Handles MoveAll.DragDelta
        If SnapToGridEnabled Then
            Margin = New Thickness(((Margin.Left + e.HorizontalChange) \ SnapPixelCountProperty) * SnapPixelCountProperty, ((Margin.Top + e.VerticalChange) \ SnapPixelCountProperty) * SnapPixelCountProperty, 0, 0)
        Else
            Margin = New Thickness(Margin.Left + e.HorizontalChange, Margin.Top + e.VerticalChange, 0, 0)
        End If

        If Me._MyObject IsNot Nothing Then
            Me._MyObject.Location = New Point(Margin.Left, Margin.Top)
        End If
    End Sub


    Private Sub UserControl_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles MyBase.Loaded

    End Sub
End Class
